package com.sun.org.apache.xml.internal.security.keys.content;

import java.security.spec.InvalidKeySpecException;
import java.security.NoSuchAlgorithmException;
import java.security.spec.X509EncodedKeySpec;
import java.security.KeyFactory;
import java.security.PublicKey;
import org.w3c.dom.Document;
import com.sun.org.apache.xml.internal.security.exceptions.XMLSecurityException;
import org.w3c.dom.Element;
import com.sun.org.apache.xml.internal.security.utils.Signature11ElementProxy;

public class DEREncodedKeyValue extends Signature11ElementProxy implements KeyInfoContent
{
    private static final String[] supportedKeyTypes;
    
    public DEREncodedKeyValue(Element element, String baseURI) throws XMLSecurityException {
        super(element, baseURI);
    }
    
    public DEREncodedKeyValue(Document doc, PublicKey publicKey) throws XMLSecurityException {
        super(doc);
        this.addBase64Text(this.getEncodedDER(publicKey));
    }
    
    public DEREncodedKeyValue(Document doc, byte[] bytes) {
        super(doc);
        this.addBase64Text(bytes);
    }
    
    public void setId(String value) {
        this.setLocalIdAttribute("Id", value);
    }
    
    public String getId() {
        return this.getLocalAttribute("Id");
    }
    
    @Override
    public String getBaseLocalName() {
        return "DEREncodedKeyValue";
    }
    
    public PublicKey getPublicKey() throws XMLSecurityException {
        byte[] bytesFromTextChild = this.getBytesFromTextChild();
        for (String algorithm : DEREncodedKeyValue.supportedKeyTypes) {
            try {
                PublicKey generatePublic = KeyFactory.getInstance(algorithm).generatePublic(new X509EncodedKeySpec(bytesFromTextChild));
                if (generatePublic != null) {
                    return generatePublic;
                }
            }
            catch (NoSuchAlgorithmException ex) {}
            catch (InvalidKeySpecException ex2) {}
        }
        throw new XMLSecurityException("DEREncodedKeyValue.UnsupportedEncodedKey");
    }
    
    protected byte[] getEncodedDER(PublicKey key) throws XMLSecurityException {
        try {
            return ((X509EncodedKeySpec)KeyFactory.getInstance(key.getAlgorithm()).getKeySpec(key, X509EncodedKeySpec.class)).getEncoded();
        }
        catch (NoSuchAlgorithmException originalException) {
            throw new XMLSecurityException(originalException, "DEREncodedKeyValue.UnsupportedPublicKey", new Object[] { key.getAlgorithm(), key.getFormat(), key.getClass().getName() });
        }
        catch (InvalidKeySpecException originalException2) {
            throw new XMLSecurityException(originalException2, "DEREncodedKeyValue.UnsupportedPublicKey", new Object[] { key.getAlgorithm(), key.getFormat(), key.getClass().getName() });
        }
    }
    
    static {
        supportedKeyTypes = new String[] { "RSA", "DSA", "EC" };
    }
}
